%% info
% This self-contained script generates Fig. 10 of our paper, 
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
%% generate figure
k0 = 2*pi/.82;  % central wavenumber in rad/um
n = 500;  % angular discretization levels

% xyz limits:
ymax = 20;
xmax = 20;
zmax = 20;

% what sizes of toruses to plot:
kmin = .8;   % these are multiples of k0
kmax = 1.2;
num_k = 5;  % number of toruses to plot

sigma_z = 1;  % axial width in um
sigma_th = .5;  % lateral width in um (for FF-OCT)

figure
for R = linspace(kmin, kmax, num_k) * k0
    
    theta_ = linspace(-pi, pi, n);
    phi_ = linspace(0, pi, n);
    [phi, theta] = meshgrid(phi_, theta_);
    
    x = (R + R*cos(theta)).*cos(phi);
    z = (R + R*cos(theta)).*sin(phi);
    y = R*sin(theta);
    
    % generate color intensity for the TF:
    kth_yz = atan2(y, z);
    kth_xz = atan2(x, z);
    
    TF_ffoct = exp(-(2*k0-2*R.*cos(kth_yz)).^2./2./(2*sigma_z*cos(kth_yz)).^2).*...
        exp(-kth_yz.^2/2/(sigma_th/2)^2);
    
    TF_pscan = exp(-(2*k0-2*R).^2./2./(2*sigma_z).^2).*...
        exp(-kth_xz.^2/2/(sigma_th/2*sqrt(2))^2);
    
    TF_tot = TF_ffoct.*TF_pscan;  % approximation for qualitative figure; it's not actually separable;
    
    TF_tot = min(TF_tot, .9)/.9;  % to adjust contrast
    
    surf(x,y,z,TF_tot, 'FaceAlpha','flat', 'alphadata', TF_tot + .1)
    daspect([1,1,1])
    shading interp
    
    hold on
    
    % plot smaller circles:
    r = 2 * R * cos(theta_-pi/2);
    Y = r.*cos(theta_);
    Z = r.*sin(theta_);
    X = 0*Y;
    plot3(X+xmax,Y,Z, '-', 'color', [.5, .5, .5])
    
    % plot larger half circles:
    theta_half = linspace(0, pi, n);
    X = 2*R*cos(theta_half);
    Z = 2*R*sin(theta_half);
    Y = 0*X;
    plot3(X,Y+ymax,Z, '-', 'color', [.5, .5, .5])
    
end

% plot labels
ylim([-ymax, ymax])
xlim([-xmax, xmax])
zlim([0, zmax])
zlabel('$k_z$ (rad/$\mu m)$', 'interpreter', 'latex')
hx = xlabel('$k_x$ (rad/$\mu m)$', 'interpreter', 'latex');
hy = ylabel('$k_y$ (rad/$\mu m)$', 'interpreter', 'latex');
hx.Position = [-1.0904 -21.9506 -3.8300];  % set manually
hy.Position = [-22.3957 -0.8638 -3.7131];
colormap(magma(512))

% plot 2D projections of the LF-OCT TF (the torus), which are the FF-OCT
% and point-scanning OCT TFs:
n = 500;
lambda = .82;
kz = linspace(0,zmax,n);  % 1/um
kx = linspace(-xmax,xmax,n);

[kz,kx] = meshgrid(kz,kx);
k0 = 2*pi/lambda;  % center wave number
kr = sqrt(kx.^2+kz.^2);
kth = atan2(kx,kz);

% TF for FF-OCT:
TF = exp(-(kr-2*k0.*cos(kth)).^2./2./(2*sigma_z*cos(kth)).^2).*...
    exp(-kth.^2/2/(sigma_th/2)^2);

inner_circle_mask = (kr - 2*k0*kmin*cos(kth)) > 0;
outer_circle_mask = (kr - 2*k0*kmax*cos(kth)) < 0;
mask = inner_circle_mask.*outer_circle_mask > 0;

surf(kx*0+xmax,kx,kz, TF, 'FaceAlpha','flat', 'alphadata', mask.*(TF+.3))
shading interp

% TF for point-scanning:
TF = exp(-(kr-2*k0).^2./2./(2*sigma_z).^2).*...
    exp(-kth.^2/2/(sigma_th/2*sqrt(2))^2);  % sqrt(2) times larger lateral resolution;

inner_circle_mask = (kr - 2*k0*kmin) > 0;
outer_circle_mask = (kr - 2*k0*kmax) < 0;
mask = inner_circle_mask.*outer_circle_mask > 0;

surf(kx,kx*0+ymax,kz, TF, 'FaceAlpha','flat', 'alphadata', mask.*(TF+.3))
shading interp