%% info
% This script generates Fig. 12 of our paper,
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
% This code requires the freezeColors.m file from MathWorks File Exchange.
% Alternatively, the line with the freezeColors command can be commented
% out, which will change some colors in the figure.
%% generate figure
theta_aper = pi/3;  % aperture half angle
k0 = 2*pi/.82;  % center wavenumber
R = k0;  % radius of circular cross-section of torus
r = R*sin(theta_aper);  % radius of the torus ring
n = 500;  % controls discretization
kmax = 1.77*k0;  % for defining plotting domain
view_angle = [-57.5, 30];  % view angle for 3D plot

% concatenate top and bottom halves
theta_ = [linspace(pi/2 - theta_aper, pi/2 + theta_aper, n), -fliplr(linspace(pi/2 - theta_aper, pi/2 + theta_aper, n))];
phi_ = [linspace(0, 2*pi, n), linspace(0, 2*pi, n)];
[phi, theta] = meshgrid(phi_, theta_);

% equations of the torus:
x = (r + R*cos(theta)).*cos(phi);
y = (r + R*cos(theta)).*sin(phi);
z = R*sin(theta);
top = (z>0);
z = z - R*cos(theta_aper)*(2*(top-.5));  % bring the toruses down to 0;

figure('position', [633 311 560 630])

% plot panel b:
subplot(212)
for i = [0, -15, -30, -45, -60]  % plot rotated toruses
    rotmat = rotx(i);
    xyz = cat(3, x, y, z);
    xyz_ = reshape(xyz, 4*n^2, 3);
    xyz_rot = xyz_ * rotmat;
    xyz_rot = reshape(xyz_rot, 2*n, 2*n, 3);
    x_new = xyz_rot(:,:,1);
    y_new = xyz_rot(:,:,2);
    z_new = xyz_rot(:,:,3);
    
    surf(x_new, y_new, z_new, z*0, 'alphadata', z*0)
    hold on
end

shading interp
camlight
daspect([1,1,1])
caxis([-.5, 1])  % select color
freezeColors  % so that we can use different color for sphere below;

theta_ = linspace(0, 2*pi, n);
phi_ = linspace(0, pi, n);
[phi, theta] = meshgrid(phi_, theta_);
X = 2*(sin(theta_aper))*R*sin(theta).*cos(phi);
Y = 2*(sin(theta_aper))*R*sin(theta).*sin(phi);
Z = 2*(sin(theta_aper))*R*cos(theta);

surf(x, y, z, z*0, 'FaceAlpha','flat','alphadata', z*0+.1);
a = surf(X,Y,Z, Z*0+1);
shading interp
alpha(a, .3);
colormap gray
freezeColors

zlabel('$k_z$ (rad/$\mu m)$', 'interpreter', 'latex')
xlabel('$k_x$ (rad/$\mu m)$', 'interpreter', 'latex')
ylabel('$k_y$ (rad/$\mu m)$', 'interpreter', 'latex')
xlim([-kmax, kmax])
ylim([-kmax, kmax])
zlim([-kmax, kmax])
view(view_angle)

title('(b) TF of multi-angle transillumination OCT', 'fontweight', 'normal')

% plot panel a:
subplot(211)
h = surf(x,y,z, z*0);
alpha(h, .5)
shading interp
camlight
colormap parula
daspect([1,1,1])
hold on

% plot 2D cross sections of panel a:
m = 9;
theta_ = [linspace(pi/2 - theta_aper, pi/2 + theta_aper, n), -fliplr(linspace(pi/2 - theta_aper, pi/2 + theta_aper, n))];
phi_ = [linspace(0, 2*pi, m), linspace(0, 2*pi, m)];
[phi, theta] = meshgrid(phi_, theta_);

x_ = (r + R*cos(theta)).*cos(phi);
y_ = (r + R*cos(theta)).*sin(phi);
z_ = R*sin(theta);
top = (z_>0);
z_ = z_ - R*cos(theta_aper)*(2*(top-.5));  % bring the tori down to 0;
for f=linspace(.7, 1, 4)
    plot3(f*x_, f*y_, f*z_, 'k')
end
xlim([-kmax, kmax])
ylim([-kmax, kmax])
zlim([-kmax, kmax])

zlabel('$k_z$ (rad/$\mu m)$', 'interpreter', 'latex')
xlabel('$k_x$ (rad/$\mu m)$', 'interpreter', 'latex')
ylabel('$k_y$ (rad/$\mu m)$', 'interpreter', 'latex')
view(view_angle)
title('(a) TF of transillumination OCT', 'fontweight', 'normal')
caxis([-.5, 1])  % select color