%% info
% This script generates Fig. 14a of our paper,
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
% This code requires the subaxis.m file from MathWorks File Exchange.
%% run simulation
n = 2048;
kmax = 300;  % max k-space extent
bandwidth = .2;  % um
NAs = [.1, .3];  % which numerical apertures to simulate
lambda = .82;  % center wavelength in um
k0 = 2*pi/lambda;  % center wavenumber

% k-space grid:
kx_ = linspace(-4*k0, 4*k0, n/4);
dx = 1/2/(max(kx_)/2/pi); % x spacing
k_sweep_ = linspace(0, kmax, n*4); % linear in k sweep
dz_sweep = 1/2/(max(k_sweep_)/2/pi);  % axial spacing
[k_sweep, kx] = meshgrid(k_sweep_, kx_);

figure('units', 'inches', 'position', [4, 4, 3.5, 2.8])
for j = 1:2  % ISAM or not
    if j==2
        kz = 2*k_sweep; % correct sampling
    else
        kz = sqrt(k_sweep.^2 - kx.^2) + k_sweep;
    end
    
    kz = real(kz);  % ignore parts of k space where kr<kx
    kth = atan2(kx, kz);
    kr = sqrt(kx.^2 + kz.^2);
    
    % define the object (sparse and denes array of point scatterers):
    obj_sparse = 0;  % only one column
    for z=-30:5:30
        obj_sparse = obj_sparse + exp(1i*(z*kz));
    end
    obj_dense = 0;
    for z=-30:5:30
        for x=-12:6:12
            obj_dense = obj_dense + exp(1i*(x*kx+z*kz));
        end
    end
    
    for i = 1:length(NAs)
        NA_FWHM = NAs(i);
        sigma_th = asin(NA_FWHM)*2/2.355;  % angle corresponding to NA_FWHM
        sigma_k = pi/sqrt(log(2))*bandwidth/lambda^2/2.355;  % bandwidth in k
        
        % transfer function:
        TF = exp(-(kr-2*k0.*cos(kth)).^2./2./(2*sigma_k*cos(kth)).^2).*... % times 2 because referencing origin
            exp(-kth.^2/2/(sigma_th/2)^2);  % divide by 2 is important -- referencing origin
        
        recon_sparse = fftshift(ifft2(ifftshift(obj_sparse.*TF)))';
        recon_dense = fftshift(ifft2(ifftshift(obj_dense.*TF)))';
        
        horizontal = (0:(length(kx_)-1))*dx;
        vertical = (0:(length(k_sweep_)-1))*dz_sweep;
        
        % plot sparse phantom:
        subaxis(length(NAs), 4, (i-1)*4+1 + j-1, 'sv', .01, 'sh', .01, 'margin', .06)
        imagesc(horizontal, vertical, abs(recon_sparse))
        if j==1
            ylabel(['NA = ', num2str(NA_FWHM)])
        end
        
        if i==1
            if j==1
                title('OCT', 'fontweight', 'normal')
            else
                title('ISAM', 'fontweight', 'normal')
            end
        end
        daspect([1,1,1])
        set(gca, 'xtick', [], 'ytick', [])
        
        % plot dense phantom:
        subaxis(length(NAs), 4, 2+ (i-1)*4+1 + j-1)
        imagesc(horizontal, vertical, abs(recon_dense))
        if i==1
            if j==1
                title('OCT', 'fontweight', 'normal')
            else
                title('ISAM', 'fontweight', 'normal')
            end
        end
        
        if i==2 && j==2
            % plot scale bar:
            len = 10;  % um
            x_max = horizontal(end);
            z_max = vertical(end);
            hold on
            plot([x_max*.91-len, x_max*.91], z_max*.97*[1,1], 'w', 'linewidth', 2)
            text(x_max*.91-len/2, z_max*.96, [num2str(len), '\mum'], 'horizontalalignment', 'center', 'verticalalignment', 'bottom','color','w', 'fontsize', 7)
        end
        daspect([1,1,1])
        set(gca, 'xtick', [], 'ytick', [])
    end
end
colormap magma
