%% info
% This script generates Fig. 17 of our paper,
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
% This code requires the legendflex.m file from MathWorks File Exchange.
% You can also comment out the legendflex command and generate the figure
% without the legend.
%% run simulations
rng(0);

lambda = .8;  % wavelength
k0 = 2*pi/lambda;  % wavenumber
n_patterns = 500;  % number of random wavefronts to average
theta_max = [2, 30, 60, 90] * pi/180;  % max angular supports to test
n_angle = 200;  % how many discretization levels over the angular range
n_dz = 500;  % how many discretization levels over the axial separation bewteen the two reflectors
dz_list = linspace(0, 2.5, n_dz);  % axial separation points
dx = 0;  % lateral separation of scatterers; keep this at 0, the worst case scenario

x = 0;
z = 0; % only look at center
psf = @(th, h, k) exp(2*1i*k0*((x-h)*sin(th)-(z-k)*cos(th)));  % oriented with fringe

color_order = get(0, 'DefaultAxesColorOrder');
figure('units', 'inches', 'position', [4 2.5938 8 5.7500])
for a = 1:4
    
    th_list = linspace(-theta_max(a), theta_max(a), n_angle);
    
    total_mod = zeros(n_dz, n_patterns, n_angle);
    for i = 1:length(dz_list)  % loop over dz separation;
        dz = dz_list(i);
        for m = 1:n_patterns  % do an INCOHERENT SUM
            for j = 1:n_angle  % here, do a COHERENT SUM instead of incoherent to form wavefront
                th = th_list(j);
                rand_phase = rand*pi*2;
                total_mod(i, m, j) = (psf(th, -dx/2, -dz/2)+psf(th, dx/2, dz/2))*exp(1i*rand_phase);
            end
        end
    end
    coherent_sum = mean(total_mod, 3);
    mod_patterns = (n_angle*abs(coherent_sum).^2 - 2)/2;  % subtract out background, normalize to 1;
    incoherent_sum = mean(mod_patterns, 2);
    
    x_axis = dz_list/lambda;
    subplot(2,2, a)
    % shade coverage region:
    CI = quantile(mod_patterns, [.025, .975], 2);  % 95% coverage
    x_ = [x_axis, fliplr(x_axis)];
    region = [CI(:, 1)', fliplr(CI(:,2)')];
    fill(x_, region, [.5, .5, .5]*1.5, 'linestyle', 'none')
    hold on
    plot(x_axis, mod_patterns(:, 1), 'linewidth', 1, 'color', color_order(1,:))  % one realization;
    plot(x_axis, incoherent_sum, 'linewidth', 2, 'color', 'k')
    plot(x_axis, cos(2*k0*dz_list), ':', 'linewidth', 2, 'color', 'c')
    plot(x_axis, besselj(0, 2*k0*dz_list), ':', 'linewidth', 2, 'color', color_order(2,:))
    if a==4  % full angular coverage
        
        title(['(d) 90',char(176),' (full) coverage'], 'fontweight', 'normal')
        legendflex({'95% pattern coverage', 'One pattern', ['Mean of ', num2str(n_patterns), ' patterns'], 'Cosine', 'Bessel'},...
            'box', 'off', 'anchor', {'ne', 'ne'}, 'xscale', .4, 'buffer', [.5, .5])
    else
        title(['(', lower(char(a + 64)), ') ', num2str(theta_max(a)*180/pi),char(176),' coverage'], 'fontweight', 'normal')
    end
    
    ylim([-1, 7])
    xlim([0, max(x_axis)])
    xlabel('$d_z/\lambda_0$', 'interpreter', 'latex', 'fontsize', 12)
end
