%% info
% This script generates Fig. 6 of our paper,
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
% This code requires the subaxis.m and arrow.m files from MathWorks File
% Exchange.
%%
lambda = .82; % um
n = 2048;
kmax = 200;
kxz = linspace(-kmax,kmax,n);  % 1/um
dxz = 2*pi/2/kmax;
[kz,kx] = meshgrid(kxz,kxz);
k0 = 2*pi/lambda;  % center wave number
kr = sqrt(kx.^2+kz.^2);  % k-space in spherical coordinates
kth = atan2(kx,kz);

bandwidths = [.15, .15];  % um
NA_FWHMs = [.05, .5];  % correpounding to FWHM of gaussian beam;

figure('units', 'inches', 'position', [4, 4, 3.5, 2.5])
for i = 1:length(bandwidths)
    bandwidth = bandwidths(i);
    NA_FWHM = NA_FWHMs(i);
    sigma_th = asin(NA_FWHM)*2/2.355;  % angle corresponding to NA_FWHM
    sigma_k = pi/sqrt(log(2))*bandwidth/lambda^2/2.355;  % from theory of OCT chapter
    
    % generate transfer function:
    TF = exp(-(kr-2*k0.*cos(kth)).^2./2./(2*sigma_k*cos(kth)).^2).*...
        exp(-kth.^2/2/(sigma_th/2)^2);
    TF = TF';
    
    th = linspace(0, 2*pi, 1000);
    circle = @(r0, radius) [radius*cos(th)+r0(1); radius*sin(th)+r0(2)];
    
    h1 = subaxis(length(bandwidths), 3, (i-1)*3+1, 'pad', 0, 'margin', .05, 'sh', .01, 'sv', .01);
    imagesc(kxz,kxz,-TF)
    if i==1
        title('(a) Transfer function', 'fontweight', 'normal')
    end
    ylabel(['NA = ', num2str(NA_FWHM)])
    kmax_plot = 2.8*k0;  % crop out borders
    xlim([-kmax_plot/2, kmax_plot/2])
    ylim([-kmax_plot/8, kmax_plot*.9])
    daspect([1, 1, 1])
    set(gca, 'ydir', 'normal', 'xtick', [], 'ytick', [])
    colormap(h1, gray(512))
    hold on
    % plot axis vectors:
    arrow([0,0], [0, kmax_plot*.2], 'length', 3,'tipangle', 30, 'width', .2, 'color', [.5, .5, .5])
    arrow([0,0], [kmax_plot*.2, 0], 'length', 3,'tipangle', 30, 'width', .2, 'color', [.5, .5, .5])
    text(0, kmax_plot*.2, '$k_z$', 'interpreter', 'latex', 'verticalalignment', 'bottom', 'horizontalalignment', 'center')
    text(kmax_plot*.2, 0, '$k_x$', 'interpreter', 'latex', 'horizontalalignment', 'left')
    text(0, 0, '$(0, 0)$', 'interpreter', 'latex', 'horizontalalignment', 'center', 'verticalalignment', 'top', 'fontsize', 8)
    % plot circles corresponding to +/- bandwidth
    c = circle([0, k0+sigma_k], k0+sigma_k);
    plot(c(1,:), c(2,:), 'b-')
    c = circle([0, k0-sigma_k], k0-sigma_k);
    plot(c(1,:), c(2,:), 'r-')
    % plot lines corresponding to NA
    x = [0, k0+sigma_k]*sin(sigma_th);
    y = x * cot(sigma_th) + k0+sigma_k;
    plot(x, y, 'b')
    plot(-x, y, 'b')
    x = [0, k0-sigma_k]*sin(sigma_th);
    y = x * cot(sigma_th) + k0-sigma_k;
    plot(x, y, 'r')
    plot(-x, y, 'r')
    
    h2 = subaxis(length(bandwidths), 3, (i-1)*3+2);
    crop = 700;  % pixels; chosen for a fixed kmax!
    PSF = fftshift(fft2(fftshift(TF)));
    peak = max(abs(PSF(:)));
    PSF = PSF/peak;  % normalize to 1
    PSF = PSF((n-crop)/2:(n+crop)/2, (n-crop)/2:(n+crop)/2);  % crop
    imagesc([0,crop*dxz],[0,crop*dxz],real(PSF));  % units of um
    if i==1
        title('(b) Re\{PSF\}', 'fontweight', 'normal')
    end
    caxis([-1,1])
    pbaspect([1, 1, 1])
    set(gca, 'ydir', 'normal', 'xtick', [], 'ytick', [])
    colormap(h2, redblue(512))
    
    h3 = subaxis(length(bandwidths), 3, (i-1)*3+3);
    imagesc([0,crop*dxz],[0,crop*dxz],-abs(PSF))
    if i==1
        title('(c) |PSF|', 'fontweight', 'normal')
    end
    caxis([-1,0])
    pbaspect([1, 1, 1])
    set(gca, 'ydir', 'normal', 'xtick', [], 'ytick', [])
    colormap(h3, gray(512))
    hold on
    
    % plot scale bar:
    if i == 2
        len = 3;  % um
        plot([crop*dxz*.95-len, crop*dxz*.95], crop*dxz*.05*[1,1], 'k', 'linewidth', 2)
        text(crop*dxz*.95-len/2, crop*dxz*.08, [num2str(len), ' \mum'], 'horizontalalignment', 'center', 'verticalalignment', 'bottom')
    end
end