%% info
% This script generates Fig. 7 of our paper,
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
% This code requires the subaxis.m and arrow.m files from MathWorks File
% Exchange.
%% plot figure where each column is a different illumination angle and each
% row is a different bandwidth/NA combo
lambda = .82; % wavelength in um
n = 512;
kmax = 20;  % max extent of k-space
kxz = linspace(-kmax, kmax, n);  % 1/um
ky = 0;  % 2D plots, so set ky to 0
dxz = 2*pi/2/kmax;  % spatial sampling
[kx, ky, kz] = meshgrid(kxz, ky, kxz);
k0 = 2*pi/lambda;  % center wave number
kr = sqrt(kx.^2+ky.^2+kz.^2);  % k-space in spherical coordinates
kph = atan2(kx,kz);
kth = acos(kz./kr);

bandwidths = [.15, .15];  % um
NA_FWHMs = [.05, .5];  % correpounding to FWHM of gaussian beam;
ill_angles = [0, 30, 60];  % illumination angles (degrees)

% for plotting the circles in the subplots:
th = linspace(0, 2*pi, 1000);
circle = @(r0, radius) [radius*cos(th)+r0(1); radius*sin(th)+r0(2)];

figure('units', 'inches', 'position', [4, 4, 3.5, 1.9958])

for i=1:length(bandwidths)
    for j=1:length(ill_angles)
        
        bandwidth = bandwidths(i);
        NA_FWHM = NA_FWHMs(i);
        ill_angle = ill_angles(j);
        
        % illumination vector:
        k_i = [sind(ill_angle), 0, cosd(ill_angle)];
        k_i = k_i/sqrt(sum(k_i.^2))*k0;  % normalize
        k_i_r = k0;  % spherical coordinates
        k_i_th = acos(k_i(end)/k_i_r);
        k_i_ph = atan2(k_i(2), k_i(1));
        k_th_i = real(acos(sum([kx(:), ky(:), kz(:)] .* k_i, 2)./kr(:)/k0));
        % ^angle between every k coordinate and illum; real part to remove parasitic imag
        k_th_i = reshape(k_th_i, size(kz));
        
        % same as above, but for the vector that has half the angle wrt to the z axis:
        k_i_half = k_i_r*[sin(k_i_th/2)*cos(k_i_ph), sin(k_i_th/2)*sin(k_i_ph), cos(k_i_th/2)];
        k_th_i_half = real(acos(sum([kx(:), ky(:), kz(:)] .* k_i_half, 2)./kr(:)/k0));
        k_th_i_half = reshape(k_th_i_half, size(kz));
        
        sigma_th = asin(NA_FWHM)*2/2.355;  % angle corresponding to NA_FWHM
        sigma_k = pi/sqrt(log(2))*bandwidth/lambda^2/2.355;  % bandwidth in k
        
        
        h1 = subaxis(length(bandwidths), length(ill_angles), (i-1)*length(ill_angles)+j, ...
            'pad', 0, 'margin', .05, 'sh', .01, 'sv', .01);
        
        % generate transfer function:
        TF = exp(-(kr-2*k0.*cos(k_th_i)).^2./2./(2*sigma_k*cos(k_th_i)).^2).*...
            exp(-(k_th_i_half).^2/2/(sigma_th/2)^2);
        TF = squeeze(TF)';
        
        imagesc(kxz,kxz,-TF)
        
        if j==1
            ylabel(['NA = ', num2str(NA_FWHM)])
        end
        if i==1
            title(['(', lower(char(j + 64)), ') ', num2str(ill_angle), char(176)], 'fontweight', 'normal')
        end
        kmax_plot = 3.5*k0;  % crop out borders
        xlim([-kmax_plot/1.7, kmax_plot/1.7])
        ylim([-kmax_plot/6, kmax_plot*.7])
        daspect([1, 1, 1])
        set(gca, 'ydir', 'normal', 'xtick', [], 'ytick', [])
        colormap(h1, gray(512))
        hold on
        
        % plot axis vectors:
        arrow([0,0], [0, kmax_plot*.2], 'length', 3,'tipangle', 30, 'width', .2, 'color', [.5, .5, .5])
        arrow([0,0], [kmax_plot*.2, 0], 'length', 3,'tipangle', 30, 'width', .2, 'color', [.5, .5, .5])
        text(0, kmax_plot*.2, '$k_z$', 'interpreter', 'latex', 'verticalalignment', 'bottom', 'horizontalalignment', 'center')
        text(kmax_plot*.2, 0, '$k_x$', 'interpreter', 'latex', 'horizontalalignment', 'left')
        text(0, 0, '$(0, 0)$', 'interpreter', 'latex', 'horizontalalignment', 'center', 'verticalalignment', 'top', 'fontsize', 8)
        % plot circles corresponding to +/- bandwidth
        c_outer = (k0+sigma_k)*[sin(k_i_th), cos(k_i_th)];  % center of inner circle
        c_inner = (k0-sigma_k)*[sin(k_i_th), cos(k_i_th)];  % center of outer circle
        c = circle(c_outer, k0+sigma_k);
        plot(c(1,:), c(2,:), 'b-')
        c = circle(c_inner, k0-sigma_k);
        plot(c(1,:), c(2,:), 'r-')
        % plot lines corresponding to NA
        x = [0, k0+sigma_k]*sin(sigma_th);
        y = x * cot(sigma_th) + k0+sigma_k;
        plot(x+c_outer(1), y+c_outer(2)-k0-sigma_k, 'b')
        plot(-x+c_outer(1), y+c_outer(2)-k0-sigma_k, 'b')
        
        x = [0, k0-sigma_k]*sin(sigma_th);
        y = x * cot(sigma_th) + k0-sigma_k;
        plot(x+c_inner(1), y+c_inner(2)-k0+sigma_k, 'r')
        plot(-x+c_inner(1), y+c_inner(2)-k0+sigma_k, 'r')
        
    end
end