%% info
% This script generates Figs. 8 and 9 of our paper,
% Kevin C. Zhou, Ruobing Qian, Al-Hafeez Dhalla, Sina Farsiu, Joseph A.
% Izatt, "Unified k-space theory of optical coherence tomography", to
% appear in Advances in Optics and Photonics.
% This code requires the subaxis.m file from MathWorks File Exchange.
% Modify the variable NA_base to toggle between Figs. 7 and 8.
%% run simulation
lambda = .82; % wavelength in um
n = 512;
kmax = 20;
kz_ = linspace(kmax/3,kmax,n);  % 1/um
kx_ = linspace(-kmax/2.1, kmax/2.1, n);
ky_ = 0;

dxz = 2*pi/2/kmax;  % spatial sampling
[kx, ky, kz] = meshgrid(kx_, ky_, kz_);
k0 = 2*pi/lambda;  % center wave number
kr = sqrt(kx.^2+ky.^2+kz.^2);  % k-space in spherical coordinates
kph = atan2(kx,kz);
kth = acos(kz./kr);

bandwidths = [.01, .30, .01, .30];  % bandwidths in um
NA_base = .5;  % .5 for figure 7, .1 for figure 8;
NA_FWHMs = [NA_base, NA_base, NA_base, NA_base];  % collection NA
NA_ills = [0, 0, NA_base, NA_base];  % illumination NA

figure('units', 'inches', 'position', [4, 4, 4, 3.22])

for i=1:4  % loop over four configs: holography, FFOCT, confocal, point-scanning OCT
    bandwidth = bandwidths(i);
    NA_FWHM = NA_FWHMs(i);
    NA_ill = NA_ills(i);
    
    sigma_th = asin(NA_FWHM)*2/2.355;  % angle corresponding to NA_FWHM
    sigma_k = pi/sqrt(log(2))*bandwidth/lambda^2/2.355;  % k-space bandwidth
    
    % integrate over many illumination k vectors
    if NA_ill == 0
        ill_sweep = 0;  % no integration
    else
        ill_sweep = linspace(-1, 1, 100)*2*k0*sin(sigma_th);
    end
    TF = 0;
    for kx_i = ill_sweep  % 2D integration or lateral k-vectors
        for ky_i = ill_sweep
            % illumination vector:
            k_i = [kx_i, ky_i, sqrt(k0^2-kx_i^2-ky_i^2)];
            k_i_r = k0;  % spherical coordinates
            k_i_th = acos(k_i(end)/k_i_r);
            k_i_ph = atan2(k_i(2), k_i(1));
            k_th_i = real(acos(sum([kx(:), ky(:), kz(:)] .* k_i, 2)./kr(:)/k0));  
            % ^angle between every k coordinate and illum; real part to remove parasitic imag
            k_th_i = reshape(k_th_i, size(kz));
            
            % same as above, but for the vector that has half the angle wrt to the z axis:
            k_i_half = k_i_r*[sin(k_i_th/2)*cos(k_i_ph), sin(k_i_th/2)*sin(k_i_ph), cos(k_i_th/2)];
            k_th_i_half = real(acos(sum([kx(:), ky(:), kz(:)] .* k_i_half, 2)./kr(:)/k0));
            k_th_i_half = reshape(k_th_i_half, size(kz));
            
            if NA_ill > 0  % only do this if the NA is actually nonzero
                gauss = exp(-(kx_i^2+ky_i^2)/2/k0^2/NA_ill^2);  % amplitude of current plane wave
            else
                gauss = 1;
            end
            
            TF = TF + gauss*exp(-(kr-2*k0.*cos(k_th_i)).^2./2./(2*sigma_k*cos(k_th_i)).^2).*...
                exp(-(k_th_i_half).^2/2/(sigma_th/2)^2);
        end
    end
    TF = squeeze(TF)'/max(TF(:));  % normalize
    
    % plot:
    h1 = subaxis(2, 2, i, 'pad', 0, 'margin', .1, 'sh', .01, 'sv', .01);
    M = contour(kx_, kz_, TF, [.5,.5]);  % to visualize width (only plot for OCT)
    imagesc(kx_, kz_, -TF)
    
    if i==1
        title('(a) Holographic microscopy', 'fontweight', 'normal')
        ylabel('$k_z$ (rad/$\mu m$)','interpreter','latex')
        set(gca, 'xtick', [])
    elseif i==2
        title('(b) Full-field OCT', 'fontweight', 'normal')
        hold on
        plot(M(1,2:end),M(2,2:end), 'r')
        set(gca, 'xtick', [], 'ytick', [])
    elseif i==3
        title('(c) Confocal microscopy', 'fontweight', 'normal')
        xlabel('$k_{xy}$  (rad/$\mu m$)','interpreter','latex')
        ylabel('$k_z$ (rad/$\mu m$)','interpreter','latex')
    else
        title('(d) Point-scanning OCT', 'fontweight', 'normal')
        hold on
        plot(M(1,2:end),M(2,2:end), 'r')
        xlabel('$k_{xy}$  (rad/$\mu m$)','interpreter','latex')
        set(gca, 'ytick', [])
    end
    
    kmax_plot = 3.5*k0;  % crop out borders
    daspect([1, 1, 1])
    set(gca, 'ydir', 'normal')

    colormap(h1, gray(512))
end